import { U32Range } from "../u32-range";
import { assert } from "../../util/assert";
export class VersionVector {
    constructor(map) {
        this.map = map;
    }
    readonlyMap() {
        return this.map;
    }
    lastSeq(replica) {
        const maybeLastSeq = this.map[replica];
        if (maybeLastSeq !== undefined) {
            return maybeLastSeq;
        }
        return 0;
    }
    isAddable(replica, seqs) {
        const lastSeq = this.lastSeq(String(replica));
        return seqs.lower <= lastSeq + 1;
    }
    addable(replica, seqs) {
        const lastSeq = this.lastSeq(String(replica));
        if (seqs.lower > lastSeq) {
            return [seqs];
        }
        else if (seqs.upper() > lastSeq) {
            const remaining = U32Range.fromBounds(lastSeq + 1, seqs.upper());
            return [remaining];
        }
        return []; // already added
    }
    add(replica, seqs) {
        assert(() => this.isAddable(replica, seqs), "seqs is addable");
        const result = this.addable(replica, seqs);
        if (result.length > 0) {
            this.map[replica] = result[0].upper();
        }
        return result;
    }
    merge(other) {
        if (other instanceof VersionVector) {
            const otherMap = other.readonlyMap();
            for (const replica in otherMap) {
                const seq = other.lastSeq(replica);
                this.map[replica] = Math.max(this.lastSeq(replica), seq);
            }
        }
        throw new Error("non-compatible implementations");
    }
}
//# sourceMappingURL=version-vector.js.map