"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var u32_range_1 = require("../u32-range");
var assert_1 = require("../../util/assert");
var VersionVector = /** @class */ (function () {
    function VersionVector(map) {
        this.map = map;
    }
    VersionVector.prototype.readonlyMap = function () {
        return this.map;
    };
    VersionVector.prototype.lastSeq = function (replica) {
        var maybeLastSeq = this.map[replica];
        if (maybeLastSeq !== undefined) {
            return maybeLastSeq;
        }
        return 0;
    };
    VersionVector.prototype.isAddable = function (replica, seqs) {
        var lastSeq = this.lastSeq(String(replica));
        return seqs.lower <= lastSeq + 1;
    };
    VersionVector.prototype.addable = function (replica, seqs) {
        var lastSeq = this.lastSeq(String(replica));
        if (seqs.lower > lastSeq) {
            return [seqs];
        }
        else if (seqs.upper() > lastSeq) {
            var remaining = u32_range_1.U32Range.fromBounds(lastSeq + 1, seqs.upper());
            return [remaining];
        }
        return []; // already added
    };
    VersionVector.prototype.add = function (replica, seqs) {
        var _this = this;
        assert_1.assert(function () { return _this.isAddable(replica, seqs); }, "seqs is addable");
        var result = this.addable(replica, seqs);
        if (result.length > 0) {
            this.map[replica] = result[0].upper();
        }
        return result;
    };
    VersionVector.prototype.merge = function (other) {
        if (other instanceof VersionVector) {
            var otherMap = other.readonlyMap();
            for (var replica in otherMap) {
                var seq = other.lastSeq(replica);
                this.map[replica] = Math.max(this.lastSeq(replica), seq);
            }
        }
        throw new Error("non-compatible implementations");
    };
    return VersionVector;
}());
exports.VersionVector = VersionVector;
//# sourceMappingURL=version-vector.js.map