function rankOf(node) {
    if (node !== undefined) {
        return node.rank;
    }
    return 0;
}
var ValuedNode = /** @class */ (function () {
    function ValuedNode(l, v, r) {
        this.left = l;
        this.value = v;
        this.right = r;
        this.rank = 0;
        this.update();
    }
    ValuedNode.leaf = function (iRange) {
        return new ValuedNode(undefined, iRange, undefined);
    };
    // Basic changes
    ValuedNode.prototype.setLeft = function (l) {
        ;
        this.left = l;
        this.update();
    };
    ValuedNode.prototype.setRight = function (r) {
        ;
        this.right = r;
        this.update();
    };
    ValuedNode.prototype.update = function () {
        this.rank = 1 + Math.max(rankOf(this.left), rankOf(this.right));
    };
    ValuedNode.prototype.updated = function () {
        return this.balance();
    };
    // Balancing
    ValuedNode.prototype.balanceFactor = function () {
        return rankOf(this.right) - rankOf(this.left);
    };
    ValuedNode.prototype.isBalanced = function () {
        var balanceFactor = this.balanceFactor();
        return -2 < balanceFactor && balanceFactor < 2;
    };
    ValuedNode.prototype.isRightUnbalanced = function () {
        return this.balanceFactor() >= 2;
    };
    ValuedNode.prototype.isRightOriented = function () {
        return this.balanceFactor() === 1;
    };
    ValuedNode.prototype.isLeftOriented = function () {
        return this.balanceFactor() <= -2;
    };
    ValuedNode.prototype.isSlighlyLeftUnbalanced = function () {
        return this.balanceFactor() === -1;
    };
    ValuedNode.prototype.rotateLeft = function () {
        var r = this.right;
        this.setRight(r.left);
        r.setLeft(this);
        return r;
    };
    ValuedNode.prototype.rotateRight = function () {
        var l = this.left;
        this.setLeft(l.right);
        l.setRight(this);
        return l;
    };
    ValuedNode.prototype.balance = function () {
        var self = this;
        while (self.isRightUnbalanced()) {
            if (self.right.isSlighlyLeftUnbalanced()) {
                self.setRight(self.right.rotateRight());
            }
            self = self.rotateLeft();
        }
        while (self.isLeftOriented()) {
            if (self.left.isRightOriented()) {
                self.setLeft(self.left.rotateLeft());
            }
            self = self.rotateRight();
        }
        return self;
    };
    // Traversal
    ValuedNode.prototype.min = function () {
        if (this.left !== undefined) {
            return this.left.min(); // tail recursion
        }
        return this.value;
    };
    ValuedNode.prototype.max = function () {
        if (this.right !== undefined) {
            return this.right.max(); // tail recursion
        }
        return this.value;
    };
    ValuedNode.prototype.predecessor = function () {
        if (this.left !== undefined) {
            return this.left.max();
        }
        return undefined;
    };
    ValuedNode.prototype.successor = function () {
        if (this.right !== undefined) {
            return this.right.min();
        }
        return undefined;
    };
    // Removal
    ValuedNode.prototype.removeMin = function () {
        if (this.left !== undefined) {
            this.setLeft(this.left.removeMin());
            return this.balance();
        }
        return this.right;
    };
    ValuedNode.prototype.removeMax = function () {
        if (this.right !== undefined) {
            this.setRight(this.right.removeMax());
            return this.balance();
        }
        return this.left;
    };
    /** @Override */
    ValuedNode.prototype.removePredecessor = function () {
        if (this.left !== undefined) {
            this.setLeft(this.left.removeMax());
        }
        // delegate balance to parent
    };
    /** @Override */
    ValuedNode.prototype.removeSuccessor = function () {
        if (this.right !== undefined) {
            this.setRight(this.right.removeMin());
        }
        // delegate balance to parent
    };
    // merge
    ValuedNode.prototype.mergeLeft = function () {
        var curr = this.value;
        var prev = this.predecessor();
        if (prev !== undefined &&
            prev.compare(curr) === -5 /* PREPENDABLE */) {
            this.removePredecessor();
            this.value = prev.append(curr);
        }
    };
    ValuedNode.prototype.mergeRight = function () {
        var curr = this.value;
        var succ = this.successor();
        if (succ !== undefined &&
            succ.compare(curr) === 5 /* APPENDABLE */) {
            this.removeSuccessor();
            this.value = curr.append(succ);
        }
    };
    // insertable
    ValuedNode.prototype.insertableLeft = function (iRange) {
        if (this.left !== undefined) {
            return this.left.insertable(iRange);
        }
        return [iRange];
    };
    ValuedNode.prototype.insertableRight = function (iRange) {
        if (this.right !== undefined) {
            return this.right.insertable(iRange);
        }
        return [iRange];
    };
    ValuedNode.prototype.insertable = function (iRange) {
        var curr = this.value;
        switch (iRange.compare(curr)) {
            case -5 /* PREPENDABLE */:
            case -6 /* BEFORE */:
                return this.insertableLeft(iRange);
            case 5 /* APPENDABLE */:
            case 6 /* AFTER */:
                return this.insertableRight(iRange);
            case 3 /* INCLUDING_RIGHT */:
            case -4 /* OVERLAPPING_BEFORE */:
                return this.insertableLeft(iRange.prependable(curr));
            case 1 /* INCLUDING_LEFT */:
            case 4 /* OVERLAPPING_AFTER */:
                return this.insertableRight(iRange.appendable(curr));
            case 2 /* INCLUDING_MIDDLE */: {
                var lIns = this.insertableLeft(iRange.prependable(curr));
                var rIns = this.insertableRight(iRange.appendable(curr));
                return lIns.concat(rIns);
            }
            case -2 /* INCLUDED_MIDDLE_BY */:
            case -1 /* INCLUDED_RIGHT_BY */:
            case -3 /* INCLUDED_LEFT_BY */:
            case 0 /* EQUAL */:
                return [];
        }
    };
    // insert
    ValuedNode.prototype.insertLeft = function (iRange) {
        var ins;
        if (this.left !== undefined) {
            ins = this.left.insert(iRange);
            this.setLeft(this.left.balance());
        }
        else {
            this.setLeft(ValuedNode.leaf(iRange));
            ins = [iRange];
        }
        // delegate balance to parent
        return ins;
    };
    ValuedNode.prototype.insertRight = function (iRange) {
        var ins;
        if (this.right !== undefined) {
            ins = this.right.insert(iRange);
            this.setRight(this.right.balance());
        }
        else {
            // should not happen
            this.setRight(ValuedNode.leaf(iRange));
            ins = [iRange];
        }
        // delegate balance to parent
        return ins;
    };
    ValuedNode.prototype.insert = function (iBlock) {
        var curr = this.value;
        switch (iBlock.compare(curr)) {
            case -6 /* BEFORE */:
                return this.insertLeft(iBlock);
            case 6 /* AFTER */:
                return this.insertRight(iBlock);
            case -5 /* PREPENDABLE */: {
                var lIns = this.insertLeft(iBlock);
                this.mergeLeft();
                return lIns;
            }
            case 5 /* APPENDABLE */: {
                var rIns = this.insertRight(iBlock);
                this.mergeRight();
                return rIns;
            }
            case -4 /* OVERLAPPING_BEFORE */:
            case 4 /* OVERLAPPING_AFTER */:
            case -2 /* INCLUDED_MIDDLE_BY */:
            case -1 /* INCLUDED_RIGHT_BY */:
            case -3 /* INCLUDED_LEFT_BY */:
            case 2 /* INCLUDING_MIDDLE */:
            case 3 /* INCLUDING_RIGHT */:
            case 1 /* INCLUDING_LEFT */:
            case 0 /* EQUAL */:
                throw new Error("inserted value intersects with one or more existing blocks");
        }
    };
    return ValuedNode;
}());
export { ValuedNode };
//# sourceMappingURL=causal-context-node.js.map